
const { done } = require("@vue/cli-shared-utils")
const merge = require('lodash.merge')

const { webpackMain } = require('../../utils')

// build
module.exports = class {
  constructor(api, opts, call) {
    const pkg = require(api.resolve("./package.json"));
    let builderConfig = merge({
      directories: {
        output: opts.builderOutputDir
      }
    }, pkg.build);
    if(typeof call === 'function') call();// 运行之前执行

    // run
    return async function(args, rawArgs) {
      process.env.IS_ELECTRON = true;
      /* 编译 renderer-process 渲染进程 */
      await api.service.run(
        "build",
        {
          ...args,
          dest: api.resolve(opts.outputDir)
        },
        rawArgs
      );
      /* 编译 main-process 主进程文件 */
      await webpackMain(api, opts);

      /* 开始编译 electron */
      if (!args.noApp) {
        buildElectron(api, {
          builderConfig
        });
      } else {
        done("Build resource, no build exe.");
      }
    }
  }
}

/**
 * 打包成 application
 * @param {*} api
 * @param {*} options
 */
function buildElectron(api, opts) {
  // 使用寄宿应用的 builder 模块
  const builder = require(api.resolve("./node_modules/electron-builder"));
  builder.build({
    config: opts.builderConfig
  }).then(() => {
    // TODO 删除 src 文件夹
    done("Build complete!");
  });
}
